@@ -263,7 +263,7 @@ sub tables
 
     $self->_check_dbh;
 
-    my @t = eval {  $self->{dbh}->tables( '', $self->{schema}->name, '%', '' ); };
+    my @t = eval {  $self->{dbh}->tables( '', '', '%', 'table' ); };
     Alzabo::Exception::Driver->throw( error => $@ ) if $@;
 
     return @t;
@@ -286,6 +286,16 @@ sub statement
 					 @_ );
 }
 
+sub statement_no_execute
+{
+    my $self = shift;
+
+    $self->_check_dbh;
+
+    return Alzabo::DriverStatement->new_no_execute( dbh => $self->{dbh},
+                                                    @_ );
+}
+
 sub func
 {
     my $self = shift;
@@ -460,6 +470,15 @@ Params::Validate::validation_options( on_fail => sub { Alzabo::Exception::Params
 $VERSION = '0.1';
 
 sub new
+{
+    my $self = shift->new_no_execute(@_);
+
+    $self->execute;
+
+    return $self;
+}
+
+sub new_no_execute
 {
     my $proto = shift;
     my $class = ref $proto || $proto;
@@ -488,8 +507,6 @@ sub new
 				      sql => $p{sql},
 				      bind => $self->{bind} ) if $@;
 
-    $self->execute;
-
     return $self;
 }
 
@@ -514,6 +531,19 @@ sub execute
 				      bind => $self->{bind} ) if $@;
 }
 
+sub execute_no_result
+{
+    my $self = shift;
+
+    eval
+    {
+	$self->{sth}->execute(@_);
+    };
+    Alzabo::Exception::Driver->throw( error => $@,
+				      sql => $self->{sth}{Statement},
+				      bind => $self->{bind} ) if $@;
+}
+
 sub next
 {
     my $self = shift;
